/**
 * Copyright (C) 2009-2013 Couchbase, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALING
 * IN THE SOFTWARE.
 */

package com.couchbase.client.vbucket.config;


import com.couchbase.client.vbucket.ConnectionException;
import java.text.ParseException;
import java.util.Map;
import junit.framework.TestCase;
import net.spy.memcached.TestConfig;

/**
 * A ConfigParserJSONTest.
 */
public class ConfigurationParserJSONTest extends TestCase {

  private final ConfigurationParser configParser = new ConfigurationParserJSON();
  private static final String DEFAULT_POOL_NAME = "default";

  /**
   * Test to check if the base string used
   * can be parsed or not.
   *
   * @pre Parse base string
   * @throws Exception the exception
   */
  public void testParseBase() throws Exception {
    Map<String, Pool> base = configParser.parsePools(BASE_STRING);
    assertNotNull(base);
    assertTrue(!base.isEmpty());
    Pool pool = base.get(DEFAULT_POOL_NAME);
    assertNotNull(pool);
    assertEquals(DEFAULT_POOL_NAME, pool.getName());
    assertNotNull(pool.getUri());
  }

  /**
   * Test to check if the buckets string used
   * can be parsed or not.
   *
   * @pre Parse bucket cluster strings
   * @post check bucket
   * @throws Exception the exception
   */
  public void testParseBuckets() throws Exception {
    Map<String, Bucket> buckets = configParser.parseBuckets(BUCKETS_STRING);
    for (Bucket bucket : buckets.values()) {
      checkBucket(bucket);
    }
  }

  /**
   * Test to check if the buckets cluster string used
   * can be parsed or not.
   *
   * @pre Parse bucket cluster strings
   * @post check bucket
   * @throws Exception the exception
   */
  public void testParseBucketsClustered() throws Exception {
    StringBuilder sb = new StringBuilder();
    sb.append(BUCKETS_CLUSTER_STRING);
    sb.append(BUCKETS_CLUSTER_STRING2);
    Map<String, Bucket> buckets = configParser.parseBuckets(sb.toString());
    for (Bucket bucket : buckets.values()) {
      checkBucket(bucket);
    }
  }

  /**
   * Test to check if the JSON bucket string used in this
   * test can be parsed into a server bucket with all the
   * configuration like URI, name etc.
   *
   * @pre Parse bucket
   * @post check bucket
   * @throws Exception the exception
   */
  public void testParseBucket() throws Exception {
    Bucket bucket = configParser.parseBucket(BUCKET_STRING);
    checkBucket(bucket);
  }

  /**
   * Test to load the JSON pool string after parsing it using the
   * configuration parser and retrieving the bucket information
   * from the created pool.
   *
   * @pre Load the pool of strings
   * @post Asserts that the bucket URI is not null
   *
   * @throws Exception the exception
   */
  public void testLoadPool() throws Exception {
    Pool pool = new Pool(null, null, null);
    configParser.parsePool(pool, POOL_STRING);
    assertNotNull(pool.getBucketsUri());
  }

  /**
   * Tests the behaviour of the client when a wrong URI is provided
   * for connection.
   *
   * @pre Wrong connection URI is passed.
   * @post Asserts equals if due to this the test runs
   * into ConnectionException.
   */
  public void testInvalidURI() throws ParseException{
    try {
      configParser.parsePools(INVALID_BASE_STRING);
    } catch (ConnectionException e) {
      assertEquals(e.getMessage(), "Connection URI is either incorrect "
        + "or invalid as it cannot be parsed.");
    }
  }

  /**
   * Check bucket.
   *
   * @param bucket the bucket
   * @throws Exception the exception
   */
  private void checkBucket(Bucket bucket) {
    assertNotNull("Bucket is null", bucket);
    assertNotNull(bucket.getName());
    assertNotNull(bucket.getStreamingURI());
    assertNotNull(bucket.getConfig());
  }

  private static final String BASE_STRING =
      "{\"pools\":[{\"name\":\"default\",\"uri\":\"/pools/default\","
          + "\"streamingUri\":\"/poolsStreaming/default\"}],\"implementation"
          + "Version\":\"1.6.0beta3\",\"componentsVersion\":{\"os_mon\":\""
          + "2.2.4\",\"mnesia\":\"4.4.12\",\"kernel\":\"2.13.4\",\"sasl\""
          + ":\"2.1.8\",\"ns_server\":\"1.6.0beta3\",\"menelaus\":\"1.6.0"
          + "beta3\",\"stdlib\":\"1.16.4\"}}";

  private static final String INVALID_BASE_STRING =
      "http://" + TestConfig.IPV4_ADDR + ":8091/index.html";

  private static final String BUCKETS_STRING = "[\n"
      + "    {\n"
      + "        \"authType\": \"sasl\", \n"
      + "        \"basicStats\": {\n"
      + "            \"diskFetches\": 0, \n"
      + "            \"diskUsed\": 2912256, \n"
      + "            \"itemCount\": 0, \n"
      + "            \"memUsed\": 26001976, \n"
      + "            \"opsPerSec\": 0, \n"
      + "            \"quotaPercentUsed\": 24.797416687011719\n"
      + "        }, \n"
      + "        \"bucketType\": \"membase\", \n"
      + "        \"flushCacheUri\": \"/pools/default/buckets/default/"
      + "controller/doFlush\", \n"
      + "        \"name\": \"default\", \n"
      + "        \"nodeLocator\": \"vbucket\", \n"
      + "        \"nodes\": [\n"
      + "            {\n"
      + "                \"clusterCompatibility\": 1, \n"
      + "                \"clusterMembership\": \"active\", \n"
      + "                \"hostname\": \"127.0.0.1:8091\", \n"
      + "                \"mcdMemoryAllocated\": 2871, \n"
      + "                \"mcdMemoryReserved\": 2871, \n"
      + "                \"memoryFree\": 193156000, \n"
      + "                \"memoryTotal\": 3764372000.0, \n"
      + "                \"os\": \"i386-apple-darwin10.6.0\", \n"
      + "                \"ports\": {\n"
      + "                    \"direct\": 11210, \n"
      + "                    \"proxy\": 11211\n"
      + "                }, \n"
      + "                \"replication\": 1.0, \n"
      + "                \"status\": \"healthy\", \n"
      + "                \"uptime\": \"57\", \n"
      + "                \"version\": \"1.6.5.3_5_gb78249f\"\n"
      + "            }\n"
      + "        ], \n"
      + "        \"proxyPort\": 0, \n"
      + "        \"quota\": {\n"
      + "            \"ram\": 104857600, \n"
      + "            \"rawRAM\": 104857600\n"
      + "        }, \n"
      + "        \"replicaNumber\": 1, \n"
      + "        \"saslPassword\": \"\", \n"
      + "        \"stats\": {\n"
      + "            \"uri\": \"/pools/default/buckets/default/stats\"\n"
      + "        }, \n"
      + "        \"streamingUri\": \"/pools/default/bucketsStreaming/"
      + "default\", \n"
      + "        \"uri\": \"/pools/default/buckets/default\", \n"
      + "        \"vBucketServerMap\": {\n"
      + "            \"hashAlgorithm\": \"CRC\", \n"
      + "            \"numReplicas\": 1, \n"
      + "            \"serverList\": [\n"
      + "                \"127.0.0.1:11210\"\n" + "            ], \n"
      + "            \"vBucketMap\": [\n" + "                [\n"
      + "                    0, \n" + "                    -1\n"
      + "                ] \n" + "            ]\n" + "        }\n"
      + "    }\n" + "]\n";

  private static final String BUCKETS_CLUSTER_STRING = "[\n"
      + "    {\n"
      + "        \"authType\": \"sasl\", \n"
      + "        \"basicStats\": {\n"
      + "            \"diskFetches\": 91, \n"
      + "            \"diskUsed\": 11399920640.0, \n"
      + "            \"itemCount\": 32112390, \n"
      + "            \"memUsed\": 11477801158.0, \n"
      + "            \"opsPerSec\": 4101, \n"
      + "            \"quotaPercentUsed\": 72.151368882753374\n"
      + "        }, \n"
      + "        \"bucketType\": \"membase\", \n"
      + "        \"flushCacheUri\": \"/pools/default/buckets/default/"
      + "controller/doFlush\", \n"
      + "        \"name\": \"default\", \n"
      + "        \"nodeLocator\": \"vbucket\", \n"
      + "        \"nodes\": [\n"
      + "            {\n"
      + "                \"clusterCompatibility\": 1, \n"
      + "                \"clusterMembership\": \"active\", \n"
      + "                \"hostname\": \"172.16.10.32:8091\", \n"
      + "                \"mcdMemoryAllocated\": 6402, \n"
      + "                \"mcdMemoryReserved\": 6402, \n"
      + "                \"memoryFree\": 1497505792, \n"
      + "                \"memoryTotal\": 8391725056.0, \n"
      + "                \"os\": \"x86_64-unknown-linux-gnu\", \n"
      + "                \"ports\": {\n"
      + "                    \"direct\": 11210, \n"
      + "                    \"proxy\": 11211\n"
      + "                }, \n"
      + "                \"replication\": 1.0, \n"
      + "                \"status\": \"healthy\", \n"
      + "                \"uptime\": \"110393\", \n"
      + "                \"version\": \"1.6.5.4\"\n"
      + "            }, \n"
      + "            {\n"
      + "                \"clusterCompatibility\": 1, \n"
      + "                \"clusterMembership\": \"active\", \n"
      + "                \"hostname\": \"172.16.10.90:8091\", \n"
      + "                \"mcdMemoryAllocated\": 12865, \n"
      + "                \"mcdMemoryReserved\": 12865, \n"
      + "                \"memoryFree\": 10322313216.0, \n"
      + "                \"memoryTotal\": 16863064064.0, \n"
      + "                \"os\": \"x86_64-unknown-linux-gnu\", \n"
      + "                \"ports\": {\n"
      + "                    \"direct\": 11210, \n"
      + "                    \"proxy\": 11211\n"
      + "                }, \n"
      + "                \"replication\": 1.0, \n"
      + "                \"status\": \"healthy\", \n"
      + "                \"uptime\": \"110422\", \n"
      + "                \"version\": \"1.6.5.4\"\n"
      + "            }, \n"
      + "            {\n"
      + "                \"clusterCompatibility\": 1, \n"
      + "                \"clusterMembership\": \"active\", \n"
      + "                \"hostname\": \"172.16.10.97:8091\", \n"
      + "                \"mcdMemoryAllocated\": 12865, \n"
      + "                \"mcdMemoryReserved\": 12865, \n"
      + "                \"memoryFree\": 10408280064.0, \n"
      + "                \"memoryTotal\": 16863092736.0, \n"
      + "                \"os\": \"x86_64-unknown-linux-gnu\", \n"
      + "                \"ports\": {\n"
      + "                    \"direct\": 11210, \n"
      + "                    \"proxy\": 11211\n"
      + "                }, \n"
      + "                \"replication\": 1.0, \n"
      + "                \"status\": \"healthy\", \n"
      + "                \"uptime\": \"110411\", \n"
      + "                \"version\": \"1.6.5.4\"\n"
      + "            }\n"
      + "        ], \n"
      + "        \"proxyPort\": 0, \n"
      + "        \"quota\": {\n"
      + "            \"ram\": 15907946496.0, \n"
      + "            \"rawRAM\": 5302648832.0\n"
      + "        }, \n"
      + "        \"replicaNumber\": 1, \n"
      + "        \"saslPassword\": \"\", \n"
      + "        \"stats\": {\n"
      + "            \"uri\": \"/pools/default/buckets/default/stats\"\n"
      + "        }, \n"
      + "        \"streamingUri\": \"/pools/default/bucketsStreaming/"
      + "default\", \n"
      + "        \"uri\": \"/pools/default/buckets/default\", \n"
      + "        \"vBucketServerMap\": {\n"
      + "            \"hashAlgorithm\": \"CRC\", \n"
      + "            \"numReplicas\": 1, \n"
      + "            \"serverList\": [\n"
      + "                \"172.16.10.32:11210\", \n"
      + "                \"172.16.10.90:11210\", \n"
      + "                \"172.16.10.97:11210\"\n" + "            ], \n"
      + "            \"vBucketMap\": [\n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n" + "                [\n"
      + "                    1, \n" + "                    2\n"
      + "                ], \n" + "                [\n"
      + "                    0, \n" + "                    1\n"
      + "                ], \n" + "                [\n"
      + "                    2, \n" + "                    0\n"
      + "                ], \n";

//overran the length of a static String!
  private static final String BUCKETS_CLUSTER_STRING2 =
      "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ], \n"
        + "                [\n" + "                    2, \n"
        + "                    0\n" + "                ], \n"
        + "                [\n" + "                    1, \n"
        + "                    2\n" + "                ], \n"
        + "                [\n" + "                    0, \n"
        + "                    1\n" + "                ]\n"
        + "            ]\n" + "        }\n" + "    }\n" + "]\n";

  private static final String POOL_STRING =
      "{\"storageTotals\":{\"ram\":{\"quotaUsed\":1073741824,"
          + "\"total\":4290801664.0,\"quotaTotal\":1073741824,\"used\":"
          + "2993655808.0},\"hdd\":{\"quotaUsed\":309237645312.0,\"usedByData"
          + "\":189196,\"total\":309417967616.0,\"quotaTotal\":309417967616.0"
          + ",\"used\":256816913121.0}},\"name\":\"default\",\"nodes\":[{\""
          + "uptime\":\"24436\",\"memoryTotal\":4290801664.0,\"memoryFree\":"
          + "1297145856,\"mcdMemoryReserved\":3273,\"mcdMemoryAllocated\":3273,"
          + "\"clusterMembership\":\"active\",\"status\":\"healthy\",\""
          + "hostname\":\"192.168.1.35:8080\",\"version\":\"1.6.0beta3\",\"os"
          + "\":\"windows\",\"ports\":{\"proxy\":11211,\"direct\":11210},\""
          + "otpNode\":\"ns_1@192.168.1.35\",\"otpCookie\":\"pbjgdbhhylfxkfol"
          + "\"}],\"buckets\":{\"uri\":\"/pools/default/buckets\"},\""
          + "controllers\":{\"addNode\":{\"uri\":\"/controller/addNode\"},"
          + "\"rebalance\":{\"uri\":\"/controller/rebalance\"},\"failOver\""
          + ":{\"uri\":\"/controller/failOver\"},\"reAddNode\":{\"uri\":\""
          + "/controller/reAddNode\"},\"ejectNode\":{\"uri\":\"/controller/"
          + "ejectNode\"},\"testWorkload\":{\"uri\":\"/pools/default/"
          + "controller/testWorkload\"}},\"balanced\":true,\"rebalanceStatus"
          + "\":\"none\",\"rebalanceProgressUri\":\"/pools/default/rebalance"
          + "Progress\",\"stopRebalanceUri\":\"/controller/stopRebalance\","
          + "\"stats\":{\"uri\":\"/pools/default/stats\"}}";

  private static final String BUCKET_STRING =
      "{\"name\":\"Administrator\",\"uri\":\"/pools/default/buckets/"
          + "Administrator\",\"streamingUri\":\"/pools/default/buckets"
          + "Streaming/Administrator\",\"nodes\":[{\"uptime\":\"24548\","
          + "\"memoryTotal\":4290801664.0,\"memoryFree\":1292242944,\""
          + "mcdMemoryReserved\":3273,\"mcdMemoryAllocated\":3273,\"cluster"
          + "Membership\":\"active\",\"status\":\"healthy\",\"hostname\":\""
          + "192.168.1.35:8080\",\"version\":\"1.6.0beta3\",\"os\":\"windows\""
          + ",\"ports\":{\"proxy\":11211,\"direct\":11210}}],\"stats\":{\"uri"
          + "\":\"/pools/default/buckets/Administrator/stats\"},\"vBucket"
          + "ServerMap\":{\"user\":\"Administrator\",\"password\":\"password\""
          + ",\"hashAlgorithm\":\"CRC\",\"numReplicas\":1,\"serverList\":[\""
          + "192.168.1.35:11210\"],\"vBucketMap\":[[0,-1]]},\"replicaNumber\""
          + ":1,\"quota\":{\"ram\":536870912,\"hdd\":307090161664.0},\"basic"
          + "Stats\":{\"opsPerSec\":0,\"diskFetches\":0,\"quotaPercentUsed\":"
          + "2.380473166704178,\"diskUsed\":94598,\"memUsed\":12780068,\"item"
          + "Count\":0}}";
}
