/**
 * Copyright (C) 2009-2013 Couchbase, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALING
 * IN THE SOFTWARE.
 */

package com.couchbase.client.vbucket.config;

import org.codehaus.jettison.json.JSONObject;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;


/**
 * Verifies the correct functionality of a {@link DefaultConfigFactory}.
 */
public class DefaultConfigFactoryTest {

  private ConfigFactory factory;

  @Before
  public void setup() {
    factory = new DefaultConfigFactory();
  }

  /**
   * Make sure that a raw config is parsed correctly.
   */
  @Test
  public void shouldCreateConfigFromJSONObject() throws Exception {
    final Config config = factory.create(new JSONObject(rawConfig1));
    assertEquals(4, config.getServers().size());
    assertEquals("192.168.56.101:11210", config.getServers().get(0));
    assertEquals("192.168.56.102:11210", config.getServers().get(1));
    assertEquals("192.168.56.103:11210", config.getServers().get(2));
    assertEquals("192.168.56.104:11210", config.getServers().get(3));

    assertEquals(1024, config.getVbuckets().size());
    assertEquals(4, config.getCouchServers().size());

    assertEquals(ConfigType.COUCHBASE, config.getConfigType());
    assertEquals(1, config.getReplicasCount());
  }

  /**
   * Make sure that a raw config is parsed correctly and a subsequent config
   * overrides the updated properties.
   */
  @Test
  public void shouldCreateConfigFromOldConfigAndJSONObject() throws Exception {
    final Config configInit = factory.create(new JSONObject(rawConfig1));
    final Config configUpdated = factory.create(new JSONObject(rawConfig2), configInit);
    final Config configInitFromUpdated = factory.create(new JSONObject(rawConfig2));

    assertEquals(3, configUpdated.getServers().size());
    assertEquals("192.168.56.101:11210", configUpdated.getServers().get(0));
    assertEquals("192.168.56.102:11210", configUpdated.getServers().get(1));
    assertEquals("192.168.56.103:11210", configUpdated.getServers().get(2));
    assertEquals(1024, configUpdated.getVbuckets().size());
    assertEquals(3, configUpdated.getCouchServers().size());

    List<VBucket> initVBuckets = configInit.getVbuckets();
    List<VBucket> updatedVBuckets = configUpdated.getVbuckets();
    List<VBucket> initUpdatedVBuckets = configInitFromUpdated.getVbuckets();
    assertEquals(initVBuckets.size(), updatedVBuckets.size());

    int sameCounter = 0;
    int changedCounter = 0;
    for (int i = 0; i < initVBuckets.size(); i++) {
      VBucket iVB = initVBuckets.get(i);
      VBucket uVB = updatedVBuckets.get(i);
      VBucket iuVB = initUpdatedVBuckets.get(i);

      if (iVB.equals(uVB)) {
        sameCounter++;
      } else {
        changedCounter++;
      }

      assertEquals(uVB.getMaster(), iuVB.getMaster());
      assertEquals(uVB.getReplica(0), iuVB.getReplica(0));
      assertEquals(uVB.getReplica(1), iuVB.getReplica(1));
      assertEquals(uVB.getReplica(2), iuVB.getReplica(2));
    }

    assertTrue(sameCounter > 0);
    assertTrue(changedCounter > 0);
  }

  /**
   * This is a 4-node config from a Couchbase Server 2.2 cluster.
   */
  private final String rawConfig1 = "{\"name\":\"default\",\"bucketType\":\"membase\",\"authType\":\"sasl\",\"saslPassword\":\"\",\"proxyPort\":0,\"replicaIndex\":false,\"uri\":\"/pools/default/buckets/default?bucket_uuid=e939c30c8c79c72611891936c595314c\",\"streamingUri\":\"/pools/default/bucketsStreaming/default?bucket_uuid=e939c30c8c79c72611891936c595314c\",\"localRandomKeyUri\":\"/pools/default/buckets/default/localRandomKey\",\"controllers\":{\"flush\":\"/pools/default/buckets/default/controller/doFlush\",\"compactAll\":\"/pools/default/buckets/default/controller/compactBucket\",\"compactDB\":\"/pools/default/buckets/default/controller/compactDatabases\",\"purgeDeletes\":\"/pools/default/buckets/default/controller/unsafePurgeBucket\",\"startRecovery\":\"/pools/default/buckets/default/controller/startRecovery\"},\"nodes\":[{\"couchApiBase\":\"http://192.168.56.104:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":5.472636815920398,\"swap_total\":805302272,\"swap_used\":26152960,\"mem_total\":1043816448,\"mem_free\":509423616},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":13470107,\"couch_docs_data_size\":13332318,\"couch_views_actual_disk_size\":75154,\"couch_views_data_size\":75154,\"curr_items\":662,\"curr_items_tot\":2510,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":33507384,\"ops\":0.0,\"vb_replica_curr_items\":1848},\"uptime\":\"25428\",\"memoryTotal\":1043816448,\"memoryFree\":509423616,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.104\",\"hostname\":\"192.168.56.104:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}},{\"couchApiBase\":\"http://192.168.56.102:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":5.0761421319796955,\"swap_total\":805302272,\"swap_used\":38731776,\"mem_total\":1043816448,\"mem_free\":442109952},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":14366914,\"couch_docs_data_size\":14228983,\"couch_views_actual_disk_size\":140792,\"couch_views_data_size\":135728,\"curr_items\":1275,\"curr_items_tot\":1933,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":33272488,\"ops\":0.0,\"vb_replica_curr_items\":658},\"uptime\":\"26393\",\"memoryTotal\":1043816448,\"memoryFree\":442109952,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.102\",\"hostname\":\"192.168.56.102:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}},{\"couchApiBase\":\"http://192.168.56.101:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":5.0761421319796955,\"swap_total\":805302272,\"swap_used\":120115200,\"mem_total\":1043816448,\"mem_free\":337616896},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":51465989,\"couch_docs_data_size\":50881338,\"couch_views_actual_disk_size\":198213,\"couch_views_data_size\":194254,\"curr_items\":1860,\"curr_items_tot\":3754,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":34309896,\"ops\":0.0,\"vb_replica_curr_items\":1894},\"uptime\":\"26738\",\"memoryTotal\":1043816448,\"memoryFree\":337616896,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.101\",\"thisNode\":true,\"hostname\":\"192.168.56.101:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}},{\"couchApiBase\":\"http://192.168.56.103:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":5.128205128205129,\"swap_total\":805302272,\"swap_used\":27779072,\"mem_total\":1043816448,\"mem_free\":444334080},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":13682494,\"couch_docs_data_size\":13545041,\"couch_views_actual_disk_size\":136610,\"couch_views_data_size\":131010,\"curr_items\":1232,\"curr_items_tot\":1861,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":33208520,\"ops\":0.0,\"vb_replica_curr_items\":629},\"uptime\":\"25903\",\"memoryTotal\":1043816448,\"memoryFree\":444334080,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.103\",\"hostname\":\"192.168.56.103:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}}],\"stats\":{\"uri\":\"/pools/default/buckets/default/stats\",\"directoryURI\":\"/pools/default/buckets/default/statsDirectory\",\"nodeStatsListURI\":\"/pools/default/buckets/default/nodes\"},\"ddocs\":{\"uri\":\"/pools/default/buckets/default/ddocs\"},\"nodeLocator\":\"vbucket\",\"fastWarmupSettings\":false,\"autoCompactionSettings\":false,\"uuid\":\"e939c30c8c79c72611891936c595314c\",\"vBucketServerMap\":{\"hashAlgorithm\":\"CRC\",\"numReplicas\":1,\"serverList\":[\"192.168.56.101:11210\",\"192.168.56.102:11210\",\"192.168.56.103:11210\",\"192.168.56.104:11210\"],\"vBucketMap\":[[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[2,3],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[1,3],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,0],[3,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,1],[3,2],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[0,3],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2],[3,2]]},\"replicaNumber\":1,\"threadsNumber\":3,\"quota\":{\"ram\":2084569088,\"rawRAM\":521142272},\"basicStats\":{\"quotaPercentUsed\":3.107131558884557,\"opsPerSec\":0.0,\"diskFetches\":0.0,\"itemCount\":132,\"diskUsed\":32522832,\"dataUsed\":32239208,\"memUsed\":64770304},\"bucketCapabilitiesVer\":\"\",\"bucketCapabilities\":[\"touch\",\"couchapi\"]}";

  /**
   * This is a 3-node config from a Couchbase Server 2.2 cluster.
   */
  private final String rawConfig2 = "{\"name\":\"default\",\"bucketType\":\"membase\",\"authType\":\"sasl\",\"saslPassword\":\"\",\"proxyPort\":0,\"replicaIndex\":false,\"uri\":\"/pools/default/buckets/default?bucket_uuid=e939c30c8c79c72611891936c595314c\",\"streamingUri\":\"/pools/default/bucketsStreaming/default?bucket_uuid=e939c30c8c79c72611891936c595314c\",\"localRandomKeyUri\":\"/pools/default/buckets/default/localRandomKey\",\"controllers\":{\"flush\":\"/pools/default/buckets/default/controller/doFlush\",\"compactAll\":\"/pools/default/buckets/default/controller/compactBucket\",\"compactDB\":\"/pools/default/buckets/default/controller/compactDatabases\",\"purgeDeletes\":\"/pools/default/buckets/default/controller/unsafePurgeBucket\",\"startRecovery\":\"/pools/default/buckets/default/controller/startRecovery\"},\"nodes\":[{\"couchApiBase\":\"http://192.168.56.101:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":6.0606060606060606,\"swap_total\":805302272,\"swap_used\":120360960,\"mem_total\":1043816448,\"mem_free\":321576960},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":59611116,\"couch_docs_data_size\":59016512,\"couch_views_actual_disk_size\":268390,\"couch_views_data_size\":262708,\"curr_items\":2542,\"curr_items_tot\":5029,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":45498808,\"ops\":0.0,\"vb_replica_curr_items\":2487},\"uptime\":\"27308\",\"memoryTotal\":1043816448,\"memoryFree\":321576960,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.101\",\"thisNode\":true,\"hostname\":\"192.168.56.101:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}},{\"couchApiBase\":\"http://192.168.56.103:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":9.0,\"swap_total\":805302272,\"swap_used\":47534080,\"mem_total\":1043816448,\"mem_free\":448458752},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":21241808,\"couch_docs_data_size\":21081341,\"couch_views_actual_disk_size\":141245,\"couch_views_data_size\":135762,\"curr_items\":1279,\"curr_items_tot\":2523,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":44013912,\"ops\":0.0,\"vb_replica_curr_items\":1244},\"uptime\":\"26473\",\"memoryTotal\":1043816448,\"memoryFree\":448458752,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.103\",\"hostname\":\"192.168.56.103:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}},{\"couchApiBase\":\"http://192.168.56.102:8092/default\",\"systemStats\":{\"cpu_utilization_rate\":5.025125628140704,\"swap_total\":805302272,\"swap_used\":46940160,\"mem_total\":1043816448,\"mem_free\":444370944},\"interestingStats\":{\"cmd_get\":0.0,\"couch_docs_actual_disk_size\":21856441,\"couch_docs_data_size\":21699747,\"couch_views_actual_disk_size\":133071,\"couch_views_data_size\":127918,\"curr_items\":1208,\"curr_items_tot\":2506,\"ep_bg_fetched\":0.0,\"get_hits\":0.0,\"mem_used\":44020488,\"ops\":0.0,\"vb_replica_curr_items\":1298},\"uptime\":\"26968\",\"memoryTotal\":1043816448,\"memoryFree\":444370944,\"mcdMemoryReserved\":796,\"mcdMemoryAllocated\":796,\"replication\":1.0,\"clusterMembership\":\"active\",\"status\":\"healthy\",\"otpNode\":\"ns_1@192.168.56.102\",\"hostname\":\"192.168.56.102:8091\",\"clusterCompatibility\":131072,\"version\":\"2.2.0-821-rel-enterprise\",\"os\":\"x86_64-unknown-linux-gnu\",\"ports\":{\"proxy\":11211,\"direct\":11210}}],\"stats\":{\"uri\":\"/pools/default/buckets/default/stats\",\"directoryURI\":\"/pools/default/buckets/default/statsDirectory\",\"nodeStatsListURI\":\"/pools/default/buckets/default/nodes\"},\"ddocs\":{\"uri\":\"/pools/default/buckets/default/ddocs\"},\"nodeLocator\":\"vbucket\",\"fastWarmupSettings\":false,\"autoCompactionSettings\":false,\"uuid\":\"e939c30c8c79c72611891936c595314c\",\"vBucketServerMap\":{\"hashAlgorithm\":\"CRC\",\"numReplicas\":1,\"serverList\":[\"192.168.56.101:11210\",\"192.168.56.102:11210\",\"192.168.56.103:11210\"],\"vBucketMap\":[[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,2],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[0,1],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,0],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[2,1],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,0],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2],[1,2]]},\"replicaNumber\":1,\"threadsNumber\":3,\"quota\":{\"ram\":1563426816,\"rawRAM\":521142272},\"basicStats\":{\"quotaPercentUsed\":4.11895966865647,\"opsPerSec\":0.0,\"diskFetches\":0.0,\"itemCount\":132,\"diskUsed\":38065866,\"dataUsed\":37826152,\"memUsed\":64396920},\"bucketCapabilitiesVer\":\"\",\"bucketCapabilities\":[\"touch\",\"couchapi\"]}";

}
